// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by a MIT-style license
// that can be found in the LICENSE file.

part of omnibus;

// Convert value to JSON compatible type.
dynamic _getJsonValue(dynamic value) {
  // Check if this is already a JSON primitive.
  if (_isJsonPrimitive(value)) {
    return value;
  } else {
    if (!_isJsonType(value)) {
      // Try again with value.toJson()
      try {
        value = value.toJson();
        // Throw error if this is still not JSON compatible.
        if (!_isJsonType(value)) {
          throw new ArgumentError.value(
              value, 'value', 'Data could not be converted to JSON data.');
        }
      } catch (e) {}
    }

    // Now value should be JSON compatible.
    if (_isJsonPrimitive(value)) {
      return value;
    } else if (value is List) {
      // Validate as list if this is a list.
      return _getJsonList(value);
    } else if (value is Map<String, dynamic>) {
      // Validate as map if this is a map.
      return _getJsonMap(value);
    } else {
      // This code should never be reached because _isJsonType is already called
      // in the try clause earlier.
      throw new ArgumentError.value(
          value, 'value', '_getJsonValue was unable to handle this value.');
    }
  }
}

/// Convert data to a list that only contains JSON data types.
List<dynamic> _getJsonList(List<dynamic> data) {
  // Iterate through list.
  for (var i = 0; i < data.length; i++) {
    // Store validated value.
    data[i] = _getJsonValue(data[i]);
  }
  return data;
}

/// Convert data to a map that only contains JSON data types.
Map<String, dynamic> _getJsonMap(Map<String, dynamic> data) {
  // Iterate through map.
  data.forEach((String key, dynamic value) {
    // Store validated value.
    try {
      data[key] = _getJsonValue(value);
    } catch (e) {}
  });
  return data;
}

/// Check if the given object is a JSON primitive (number, boolean, null,
/// string). This code is derived from dart:convert's json.dart
bool _isJsonPrimitive(dynamic object) {
  return (object is num ||
      identical(object, true) ||
      identical(object, false) ||
      object == null ||
      object is String);
}

/// Check if the given object is a JSON compatible type (number, boolean, null,
/// string). This code is derived from dart:convert's json.dart
bool _isJsonType(dynamic object) {
  return (object is num ||
      identical(object, true) ||
      identical(object, false) ||
      object == null ||
      object is String ||
      object is List ||
      object is Map<String, dynamic>);
}
