// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by a MIT-style license
// that can be found in the LICENSE file.

library omnibus.test.conditions_test;

import 'dart:async';

import 'package:test/test.dart';
import 'package:omnibus/omnibus.dart';

import 'messages.dart';

main() {
  var bus = new Omnibus();

  test('Bind null handler', () {
    bool caught = false;
    try {
      bus.bindEventHandler(mDataId, {}, null);
    } on ArgumentError catch (e) {
      caught = e.name == 'handler';
    }
    expect(caught, equals(true));
  });

  test('Publish events while there are no handlers', () {
    // Event with conditions
    bus.publishEvent(mData({'a': 'a'}));

    // Event without conditions
    bus.publishEvent(mData({}));
  });

  test('Various conditions filtering', () {
    // Bind some handlers.
    bus.bindEventHandler(mDataId, {'a': 'a', 'b': 'b', 'c': 'c'}, dummyHandler);
    bus.bindEventHandler(mDataId, {'a': 'a', 'b': 'b', 'c': 'a'}, dummyHandler);
    bus.bindEventHandler(mDataId, {'a': 'b', 'b': 'b', 'c': 'a'}, dummyHandler);

    // Publish some events.
    bus.publishEvent(mData({'a': 'a', 'b': 'b', 'c': 'c'}));
  });

  test('Condition-less handlers', () {
    bus.bindEventHandler(mForEveryoneId, {}, mForEveryoneHandler);
    bus.publishEvent(mForEveryone(message: 'This message is for everyone!'));
  });
}

Future dummyHandler(Map<String, dynamic> data) async {
  expect(data['a'], equals('a'));
  expect(data['b'], equals('b'));
  expect(data['c'], equals('c'));
}

Future mForEveryoneHandler(Map<String, dynamic> data) async {
  expect(data['message'], equals('This message is for everyone!'));
}
