// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by a MIT-style license
// that can be found in the LICENSE file.

library omnibus.test.event_test;

import 'dart:async';

import 'package:test/test.dart';
import 'package:omnibus/omnibus.dart';

import 'messages.dart';

/// This variable is used to check that events are indeed processed
/// asynchronously.
var asyncSwitch = true;

main() {
  test('Send MEmail message', () {
    var bus = new Omnibus();
    bus.bindEventHandler(mEmailId, {'to': 'me@example.com'}, emailHandler);

    // Process asynchronous.
    bus.publishEvent(
        mEmail(from: 'bad@example.com', to: 'notme@example.com', body: 'Hi!'));
    bus.publishEvent(
        mEmail(from: 'good@example.com', to: 'me@example.com', body: 'Hi!'));
    asyncSwitch = false;

    // Await event using AsyncProcesses<Null>.
    var pending = bus.publishEvent(
        mEmail(from: 'good@example.com', to: 'me@example.com', body: 'Hi!'));
    pending.then((AsyncProcesses<Null> processes) async {
      await processes.onComplete(() {
        asyncSwitch = true;
      });

      // Check if onComplete was executed.
      expect(asyncSwitch, equals(true));
    });
  });
}

Future emailHandler(Map<String, dynamic> email) async {
  // print("You've Got Mail!");
  // print("From:\t${email['from']}");
  // print("To:\t${email['to']}");
  // print("Body:\t${email['body']}");
  expect(asyncSwitch, equals(false));
  expect(email['to'], equals('me@example.com'));
}
