// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by a MIT-style license
// that can be found in the LICENSE file.

library omnibus.test.messages;

import 'package:omnibus/omnibus.dart';

/// Omnibus ids
const String mDataId = 'omnibus.mData';
const String mEmailId = 'omnibus.mEmail';
const String mForEveryoneId = 'omnibus.mForEveryone';
const String mCalculationRequestId = 'omnibus.mCalculationRequest';
const String mCalculationResponseId = 'omnibus.mCalculationResponse';

/// General purpose event to test conditions
OmnibusEvent mData(Map<String, dynamic> data) {
  // Do not use unchecked since we cannot know if the input Map only contains
  // JSON compatible data types.
  return new OmnibusEvent(mDataId, data);
}

/// Email event message
OmnibusEvent mEmail({String from: '', String to: '', String body: ''}) {
  return new OmnibusEvent.unchecked(
      mEmailId, {'from': from, 'to': to, 'body': body});
}

/// Condition-less event to test conditions.
OmnibusEvent mForEveryone({String message: ''}) {
  return new OmnibusEvent.unchecked(mForEveryoneId, {'message': message});
}

/// Simple calculation types
enum CalculationOperator { add, subtract, multiply, divide }

/// Calculation request message
OmnibusRequest mCalculationRequest(
    {double a: null,
    double b: null,
    CalculationOperator op: CalculationOperator.add}) {
  return new OmnibusRequest.unchecked(
      mCalculationRequestId, {'a': a, 'b': b, 'op': op.index});
}

/// Storage class for the calculation result to test [toJson]
class CalculationResult {
  final double result;
  CalculationResult(this.result) {}
  double toJson() => result;
}

/// Calculation request response
OmnibusResponse mCalculationResponse({CalculationResult result: null}) {
  return new OmnibusResponse(mCalculationResponseId, {'result': result});
}
