// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by a MIT-style license
// that can be found in the LICENSE file.

library omnibus.test.request_test;

import 'dart:async';

import 'package:test/test.dart';
import 'package:omnibus/omnibus.dart';

import 'messages.dart';

main() {
  test('Run calculations using MCalculationRequest messages', () async {
    var bus = new Omnibus();

    // Bind calculation handlers.
    bus.bindRequestHandler(mCalculationRequestId,
        {'op': CalculationOperator.add.index}, calcAddHandler);
    bus.bindRequestHandler(mCalculationRequestId,
        {'op': CalculationOperator.subtract.index}, calcSubtractHandler);
    bus.bindRequestHandler(mCalculationRequestId,
        {'op': CalculationOperator.multiply.index}, calcMultiplyHandler);
    bus.bindRequestHandler(mCalculationRequestId,
        {'op': CalculationOperator.divide.index}, calcDivideHandler);

    // Request addition.
    var responses = await bus.publishRequest(
        mCalculationRequest(a: 1.0, b: 1.0, op: CalculationOperator.add));
    responses.onComplete(() {
      expect(responses.count, equals(1));
      responses.forEach(mCalculationResponseId,
          (Map<String, dynamic> response) {
        expect(response['result'], equals(2.0));
      });
    });

    // Request division.
    // This time do not use onComplete but run forEach immediately. In a
    // real-life scenario this can increase the overall performance since the
    // responses that are already collected can be processed while the remaining
    // responses are collected. This part also uses the Future API rather than
    // the await alias.
    bus
        .publishRequest(mCalculationRequest(
            a: 12.0, b: 8.0, op: CalculationOperator.divide))
        .then((AsyncResponses responses) {
      responses.forEach(mCalculationResponseId,
          (Map<String, dynamic> response) {
        expect(response['result'], equals(1.5));
      });
    });
  });
}

Future<OmnibusResponse> calcAddHandler(Map<String, dynamic> input) async {
  return mCalculationResponse(
      result: new CalculationResult(input['a'] + input['b']));
}

Future<OmnibusResponse> calcSubtractHandler(Map<String, dynamic> input) async {
  return mCalculationResponse(
      result: new CalculationResult(input['a'] - input['b']));
}

Future<OmnibusResponse> calcMultiplyHandler(Map<String, dynamic> input) async {
  return mCalculationResponse(
      result: new CalculationResult(input['a'] * input['b']));
}

Future<OmnibusResponse> calcDivideHandler(Map<String, dynamic> input) async {
  // Delay by 1 second to run through future callbacks in Responses.forEach.
  return new Future.delayed(
      new Duration(seconds: 1),
      () => mCalculationResponse(
          result: new CalculationResult(input['a'] / input['b'])));
}
